var EPSILON = {0},
  DODGE_DISTANCE = {1};

function distance2D(a, b) {{
  var R = 6371e3, // meters
    la1 = a.latitude * Math.PI / 180, // lat, lon in radians
    la2 = b.latitude * Math.PI / 180,
    lo1 = a.longitude * Math.PI / 180,
    lo2 = b.longitude * Math.PI / 180,
    haversine_phi = Math.pow(Math.sin((la2 - la1) / 2), 2),
    sin_lon = Math.sin((lo2 - lo1) / 2),
    h = haversine_phi + Math.cos(la1) * Math.cos(la2) * sin_lon * sin_lon;
  return 2 * R * Math.asin(Math.sqrt(h));
}}

function distance(a, b) {{
  return Math.sqrt(
    Math.pow(a.altitude - b.altitude, 2) + Math.pow(distance2D(a, b), 2)
  );
}}

me.onStart = function (timestamp) {{
  me.direction_set = false;
  me.dodging = false;
  me.ongoing_detection = false;
  me.takeOff();
}};

me.onGetMsg = function (msg) {{
  if (msg && msg.flag_positions) {{
    me.flag_positions = msg.flag_positions;
    me.next_checkpoint = me.id % me.flag_positions.length;
  }}
}};

me.onUpdate = function (timestamp) {{
  if (!me.isReadyToFly()) {{
    return;
  }}
  if (!me.flag_positions) return;
  if (me.dodging) {{
    me.current_position = me.getCurrentPosition();
    var dist = distance(
      me.current_position,
      me.dodging.position
    );
    if (dist >= DODGE_DISTANCE) {{
      me.dodging = false;
    }}
    return;
  }}
  if (!me.direction_set) {{
    if (me.next_checkpoint < me.flag_positions.length) {{
      me.setTargetCoordinates(
        me.flag_positions[me.next_checkpoint].position.latitude,
        me.flag_positions[me.next_checkpoint].position.longitude,
        me.flag_positions[me.next_checkpoint].position.altitude + me.id,
        {2}
      );
    }}
    me.direction_set = true;
    return;
  }}
  if (me.next_checkpoint < me.flag_positions.length) {{
    if (!me.ongoing_detection) {{
      me.getDroneViewInfo();
      me.ongoing_detection = true;
    }}
  }}
  if (me.next_checkpoint < me.flag_positions.length) {{
    me.current_position = me.getCurrentPosition();
    me.distance = distance(
      me.current_position,
      me.flag_positions[me.next_checkpoint].position
    );
    if (me.distance <= EPSILON) {{
      me.next_checkpoint += 1;
      me.direction_set = false;
    }}
    return;
  }}
}};

me.onDroneViewInfo = function (drone_view) {{
  me.ongoing_detection = false;
  if (drone_view && drone_view.obstacles && drone_view.obstacles.length) {{
    me.dodging = drone_view.obstacles[0];
    me.direction_set = false;
    var random = Math.random() < 0.5, dodge_point = {{}};
    Object.assign(dodge_point, me.flag_positions[me.next_checkpoint].position);
    if (random) {{
      dodge_point.latitude = dodge_point.latitude * -1;
    }} else {{
      dodge_point.longitude = dodge_point.longitude * -1;
    }}
    me.setTargetCoordinates(dodge_point.latitude, dodge_point.longitude, me.getCurrentPosition().altitude, 16);
    return;
  }}
}};